import datetime
import time
from itertools import cycle
from threading import Thread
from colorama import Fore


class Loader:
    """
    Animated terminal spinner / loading indicator.

    Usage:
        # As context manager (recommended)
        with Loader(desc="Fetching data", end="Done!"):
            time.sleep(3)

        # Manual control
        loader = Loader(desc="Processing")
        loader.start()
        time.sleep(3)
        loader.stop()
    """

    PINK           = "\033[38;5;176m"
    MAGENTA        = "\033[38;5;97m"
    BRIGHT_MAGENTA = "\033[38;2;157;38;255m"
    GREEN          = "\033[38;5;40m"

    FRAMES = ["⢿", "⣻", "⣽", "⣾", "⣷", "⣯", "⣟", "⡿"]

    def __init__(
        self,
        prefix: str = "karenhoyoshi.asia",
        desc: str = "Loading...",
        end: str = "\r",
        timeout: float = 0.1,
    ):
        self.prefix = prefix
        self.desc = desc
        self.end = end
        self.timeout = timeout
        self.done = False
        self._thread = Thread(target=self._animate, daemon=True)

    # ── Context manager ───────────────────────────────────────────────────────

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.stop()

    # ── Control ───────────────────────────────────────────────────────────────

    def start(self):
        self.done = False
        self._thread = Thread(target=self._animate, daemon=True)
        self._thread.start()
        return self

    def stop(self):
        self.done = True
        self._thread.join()
        if self.end != "\r":
            current_time = datetime.datetime.now().strftime("%H:%M:%S")
            print(
                f"\n{self.PINK}[{self.MAGENTA}{self.prefix}{self.PINK}] "
                f"[{self.BRIGHT_MAGENTA}{current_time}{self.PINK}] "
                f"{self.GREEN} {self.end} {Fore.RESET}",
                flush=True,
            )
        else:
            print("\r" + " " * 80 + "\r", end="", flush=True)

    # ── Internal ──────────────────────────────────────────────────────────────

    def _animate(self):
        for frame in cycle(self.FRAMES):
            if self.done:
                break
            current_time = datetime.datetime.now().strftime("%H:%M:%S")
            print(
                f"\r{self.PINK}[{self.MAGENTA}{self.prefix}{self.PINK}] "
                f"[{self.BRIGHT_MAGENTA}{current_time}{self.PINK}] "
                f"[{self.GREEN}{self.desc}{self.PINK}]{Fore.RESET} {frame}",
                flush=True,
                end="",
            )
            time.sleep(self.timeout)



import os
import getpass
import shutil
from colorama import Fore, Style


class Home:
    """
    Displays a styled ASCII banner / home screen in the terminal.

    Usage:
        home = Home(
            title="MYAPP",
            subtitle="The coolest tool ever",
            credits="by karenhoyoshi",
            adinfo1="v1.0.0",
            adinfo2="github.com/karen",
        )
        home.display()
    """

    # Gradient colors (ANSI 256)
    _GRADIENT = [
        "\033[38;5;196m",  # red
        "\033[38;5;202m",
        "\033[38;5;208m",
        "\033[38;5;214m",
        "\033[38;5;220m",
        "\033[38;5;226m",
        "\033[38;5;190m",
        "\033[38;5;154m",
        "\033[38;5;118m",
        "\033[38;5;82m",   # green
        "\033[38;5;46m",
        "\033[38;5;47m",
        "\033[38;5;48m",
        "\033[38;5;51m",   # cyan
        "\033[38;5;45m",
        "\033[38;5;39m",
        "\033[38;5;33m",
        "\033[38;5;27m",   # blue
        "\033[38;5;57m",
        "\033[38;5;93m",   # purple
    ]

    def __init__(
        self,
        title: str = "LOGKAREN",
        subtitle: str = "Advanced Python Logger",
        credits: str | None = None,
        adinfo1: str | None = None,
        adinfo2: str | None = None,
        clear: bool = True,
    ):
        self.title = title
        self.subtitle = subtitle
        self.credits = credits
        self.adinfo1 = adinfo1
        self.adinfo2 = adinfo2
        self.clear = clear
        self.username = getpass.getuser()

    # ── Public ────────────────────────────────────────────────────────────────

    def display(self):
        if self.clear:
            os.system("cls" if os.name == "nt" else "clear")

        width = shutil.get_terminal_size().columns

        self._print_banner(width)
        self._print_subtitle(width)
        self._print_divider(width)
        self._print_welcome(width)
        self._print_adinfo(width)
        self._print_divider(width)
        print()

    # ── Internal ──────────────────────────────────────────────────────────────

    def _gradient_line(self, line: str, index: int) -> str:
        color = self._GRADIENT[index % len(self._GRADIENT)]
        return f"{color}{line}{Style.RESET_ALL}"

    def _print_banner(self, width: int):
        # Big block-letter banner using box-drawing characters
        lines = self._build_big_text(self.title)
        for i, line in enumerate(lines):
            centered = line.center(width)
            print(self._gradient_line(centered, i))

    def _print_subtitle(self, width: int):
        print()
        subtitle_colored = f"\033[38;5;141m{self.subtitle.center(width)}{Style.RESET_ALL}"
        print(subtitle_colored)
        print()

    def _print_divider(self, width: int):
        divider = "═" * width
        print(self._gradient_line(divider, 5))

    def _print_welcome(self, width: int):
        welcome = f"Welcome, {self.username}"
        if self.credits:
            welcome += f"  |  {self.credits}"
        tilde = "~" * len(welcome)

        print(self._gradient_line(welcome.center(width), 10))
        print(self._gradient_line(tilde.center(width), 12))

    def _print_adinfo(self, width: int):
        parts = [p for p in [self.adinfo1, self.adinfo2] if p]
        if not parts:
            return
        info_str = "   ·   ".join(parts)
        print(self._gradient_line(info_str.center(width), 8))

    @staticmethod
    def _build_big_text(text: str) -> list[str]:
        """Render text using hardcoded box-drawing ASCII art."""
        BANNER = r"""
██╗  ██╗ █████╗ ██████╗ ███████╗███╗   ██╗██╗███████╗███╗   ███╗███████╗
██║ ██╔╝██╔══██╗██╔══██╗██╔════╝████╗  ██║██║██╔════╝████╗ ████║██╔════╝
█████╔╝ ███████║██████╔╝█████╗  ██╔██╗ ██║██║███████╗██╔████╔██║█████╗  
██╔═██╗ ██╔══██║██╔══██╗██╔══╝  ██║╚██╗██║██║╚════██║██║╚██╔╝██║██╔══╝  
██║  ██╗██║  ██║██║  ██║███████╗██║ ╚████║██║███████║██║ ╚═╝ ██║███████╗
╚═╝  ╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═══╝╚═╝╚══════╝╚═╝     ╚═╝╚══════╝"""
        return [line for line in BANNER.splitlines() if line.strip()]


"""
logkaren — A beautiful Python logging library.

Quick start:
    from logkaren import Logger, Loader, Home, LogLevel

    log = Logger()
    log.info("Hello, world!")
    log.success("Task completed", start=0, end=1.23)
    log.warning("Be careful!")
    log.error("Something went wrong")
    log.debug("Debug info here")

    with Loader(desc="Processing"):
        import time; time.sleep(2)

    Home(title="MYAPP", subtitle="My cool tool").display()
"""

from .enums import LogLevel
from .logger import Logger, ColorLogger, SimpleLogger
from .loader import Loader
from .home import Home

__all__ = [
    "Logger",
    "ColorLogger",
    "SimpleLogger",
    "Loader",
    "Home",
    "LogLevel",
]

__version__ = "1.0.0"
__author__ = "karenhoyoshi"
__email__ = ""
__url__ = "https://github.com/karenhoyoshi/logkaren"



import datetime
import os
import re
from colorama import Fore, Style, init
from .enums import LogLevel

init(autoreset=False)


class Logger:
    """
    Base Logger factory class.

    Usage:
        log = Logger()                        # ColorLogger (default)
        log = Logger(style=2)                 # SimpleLogger
        log = Logger(prefix="myapp")          # Custom prefix
        log = Logger(level=LogLevel.WARNING)  # Only WARNING and above
        log = Logger(log_file="logs/app.log") # Save logs to file
    """

    def __new__(cls, style: int = 1, *args, **kwargs):
        if cls is Logger:
            if style == 2:
                return super().__new__(SimpleLogger)
            return super().__new__(ColorLogger)
        return super().__new__(cls)

    def __init__(
        self,
        style: int = 1,
        prefix: str | None = "karenhoyoshi.asia",
        github_repository: str | None = None,
        level: LogLevel = LogLevel.DEBUG,
        log_file: str | None = None,
    ):
        self.level = level
        self.repo_url = github_repository
        self.log_file = log_file
        self.prefix = prefix

        if log_file:
            os.makedirs(os.path.dirname(log_file), exist_ok=True) if os.path.dirname(log_file) else None
            self._write_to_log(f"=== Logging started at {datetime.datetime.now()} ===\n")

    # ── Helpers ──────────────────────────────────────────────────────────────

    def get_time(self) -> str:
        return datetime.datetime.now().strftime("%H:%M:%S")

    def _should_log(self, message_level: LogLevel) -> bool:
        return message_level.value >= self.level.value

    def _strip_ansi(self, text: str) -> str:
        ansi_escape = re.compile(r"\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])")
        return ansi_escape.sub("", text)

    def _write_to_log(self, message: str) -> None:
        if self.log_file:
            try:
                with open(self.log_file, "a", encoding="utf-8") as f:
                    f.write(self._strip_ansi(message) + "\n")
            except Exception as e:
                print(f"Error writing to log file: {e}")

    def _extract_github_username(self, url: str) -> str | None:
        url = url.replace("https://", "").replace("http://", "").replace("www.", "")
        patterns = [
            r"^github\.com/([^/]+)(?:/.*)?$",
            r"^([^/]+)(?:/.*)?$",
            r"^@(.+)$",
        ]
        for pattern in patterns:
            if match := re.search(pattern, url):
                return match.group(1).rstrip("/ \t\n\r")
        return None

    def display_repo_info(self):
        if self.repo_url:
            username = self._extract_github_username(self.repo_url)
            if username:
                self.info(f"Developed by {username} - {self.repo_url}")
            else:
                self.info(f"GitHub Repository: {self.repo_url}")

    # ── Abstract-like methods (overridden by subclasses) ──────────────────────

    def info(self, message: str, start=None, end=None): ...
    def debug(self, message: str, start=None, end=None): ...
    def warning(self, message: str, start=None, end=None): ...
    def success(self, message: str, start=None, end=None): ...
    def failure(self, message: str, start=None, end=None): ...
    def error(self, message: str, start=None, end=None): ...
    def critical(self, message: str, start=None, end=None, exit_code: int = 1): ...
    def question(self, message: str) -> str: ...


# ── ColorLogger ───────────────────────────────────────────────────────────────

class ColorLogger(Logger):
    """Full-color logger with timestamp and styled prefix."""

    # ANSI color constants
    WHITE          = "\u001b[37m"
    MAGENTA        = "\033[38;5;97m"
    BRIGHT_MAGENTA = "\033[38;2;157;38;255m"
    LIGHT_CORAL    = "\033[38;5;210m"
    RED            = "\033[38;5;196m"
    GREEN          = "\033[38;5;40m"
    YELLOW         = "\033[38;5;220m"
    BLUE           = "\033[38;5;21m"
    PINK           = "\033[38;5;176m"
    CYAN           = "\033[96m"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        raw_prefix = self.prefix or ""
        self.prefix = f"{self.PINK}[{self.MAGENTA}{raw_prefix}{self.PINK}] " if raw_prefix else f"{self.PINK}"
        self.display_repo_info()

    # ── Internal formatters ───────────────────────────────────────────────────

    def _timer_str(self, start, end) -> str:
        if start is not None and end is not None:
            elapsed = str(float(end) - float(start))[:5]
            return f" {self.BRIGHT_MAGENTA}In{self.WHITE} -> {self.BRIGHT_MAGENTA}{elapsed} Seconds{Fore.RESET}"
        return ""

    def _base(self, level_str: str, message_str: str, start=None, end=None) -> str:
        t = self.get_time()
        timer = self._timer_str(start, end)
        return (
            f"{self.prefix}"
            f"[{self.BRIGHT_MAGENTA}{t}{self.PINK}] "
            f"{self.PINK}[{level_str}{self.PINK}] -> "
            f"{message_str}{Fore.RESET}"
            f"{timer}"
        )

    def _print_and_log(self, msg: str):
        print(msg)
        self._write_to_log(msg)

    # ── Public log methods ────────────────────────────────────────────────────

    def info(self, message: str, start=None, end=None) -> None:
        if self._should_log(LogLevel.INFO):
            msg = self._base(f"{Fore.BLUE}!", f"{self.CYAN}{message}", start, end)
            self._print_and_log(msg)

    def debug(self, message: str, start=None, end=None) -> None:
        if self._should_log(LogLevel.DEBUG):
            msg = self._base(f"{Fore.YELLOW}DEBUG", f"{self.GREEN}{message}", start, end)
            self._print_and_log(msg)

    def success(self, message: str, start=None, end=None, level: str = "Success") -> None:
        if self._should_log(LogLevel.SUCCESS):
            msg = self._base(f"{self.GREEN}{level}", f"{self.GREEN}{message}", start, end)
            self._print_and_log(msg)

    def failure(self, message: str, start=None, end=None, level: str = "Failure") -> None:
        if self._should_log(LogLevel.FAILURE):
            msg = self._base(f"{self.RED}{level}", f"{self.RED}{message}", start, end)
            self._print_and_log(msg)

    def error(self, message: str, start=None, end=None, level: str = "Error") -> None:
        if self._should_log(LogLevel.FAILURE):
            msg = self._base(f"{self.RED}{level}", f"{self.RED}{message}", start, end)
            self._print_and_log(msg)

    def warning(self, message: str, start=None, end=None, level: str = "Warning") -> None:
        if self._should_log(LogLevel.WARNING):
            msg = self._base(f"{self.YELLOW}{level}", f"{self.YELLOW}{message}", start, end)
            self._print_and_log(msg)

    def critical(self, message: str, start=None, end=None, level: str = "CRITICAL", exit_code: int = 1) -> None:
        if self._should_log(LogLevel.CRITICAL):
            t = self.get_time()
            timer = self._timer_str(start, end)
            msg = (
                f"{self.prefix}"
                f"[{self.BRIGHT_MAGENTA}{t}{self.PINK}]{Fore.RESET} "
                f"{self.PINK}[{self.RED}{level}{self.PINK}] -> "
                f"{self.LIGHT_CORAL}{message}{Fore.RESET}{timer}"
            )
            print(msg)
            input()
            self._write_to_log(msg)
            self._write_to_log(f"=== Program terminated with exit code {exit_code} at {datetime.datetime.now()} ===")
            exit(exit_code)

    def question(self, message: str) -> str:
        t = self.get_time()
        prompt = (
            f"{self.prefix}"
            f"[{self.BRIGHT_MAGENTA}{t}{self.PINK}]{Fore.RESET} "
            f"{self.PINK}[{Fore.BLUE}?{self.PINK}] ->  "
            f"{self.CYAN}{message}{Fore.RESET}"
        )
        print(prompt, end="")
        answer = input()
        self._write_to_log(prompt)
        self._write_to_log(f"User Answer: {answer}")
        return answer

    def message(self, level: str, message: str, start=None, end=None) -> None:
        timer = self._timer_str(start, end)
        t = self.get_time()
        msg = (
            f"{self.prefix}"
            f"[{self.BRIGHT_MAGENTA}{t}{self.PINK}] "
            f"[{self.CYAN}{level}{self.PINK}] -> "
            f"[{self.CYAN}{message}{self.PINK}]{timer}"
        )
        print(msg)
        self._write_to_log(msg)


# ── SimpleLogger ──────────────────────────────────────────────────────────────

class SimpleLogger(Logger):
    """Minimal, clean logger without heavy styling."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._raw_prefix = self.prefix or ""
        self.prefix = f"{Fore.BLACK}{self.get_time()} » {Fore.RESET}"
        self.display_repo_info()

    def _timer_str(self, start, end) -> str:
        if start is not None and end is not None:
            return f" (In {str(float(end) - float(start))[:5]}s)"
        return ""

    def _print_and_log(self, msg: str):
        print(msg)
        self._write_to_log(msg)

    def info(self, message: str, start=None, end=None, level: str = "INFO") -> None:
        if self._should_log(LogLevel.INFO):
            msg = f"{self.prefix}{Fore.LIGHTBLUE_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            self._print_and_log(msg)

    def debug(self, message: str, start=None, end=None) -> None:
        if self._should_log(LogLevel.DEBUG):
            msg = f"{self.prefix}{Fore.GREEN}[{Fore.YELLOW}DEBUG{Fore.GREEN}]{Fore.BLACK} ➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            self._print_and_log(msg)

    def success(self, message: str, start=None, end=None, level: str = "SUCCESS") -> None:
        if self._should_log(LogLevel.SUCCESS):
            msg = f"{self.prefix}{Fore.LIGHTGREEN_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            self._print_and_log(msg)

    def failure(self, message: str, start=None, end=None, level: str = "FAILURE") -> None:
        if self._should_log(LogLevel.FAILURE):
            msg = f"{self.prefix}{Fore.LIGHTRED_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            self._print_and_log(msg)

    def error(self, message: str, start=None, end=None, level: str = "ERROR") -> None:
        if self._should_log(LogLevel.FAILURE):
            msg = f"{self.prefix}{Fore.LIGHTRED_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            self._print_and_log(msg)

    def warning(self, message: str, start=None, end=None, level: str = "WARNING") -> None:
        if self._should_log(LogLevel.WARNING):
            msg = f"{self.prefix}{Fore.LIGHTYELLOW_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            self._print_and_log(msg)

    def critical(self, message: str, start=None, end=None, level: str = "CRITICAL", exit_code: int = 1) -> None:
        if self._should_log(LogLevel.CRITICAL):
            msg = f"{self.prefix}{Fore.RED}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
            print(msg)
            input()
            self._write_to_log(msg)
            self._write_to_log(f"=== Program terminated with exit code {exit_code} at {datetime.datetime.now()} ===")
            exit(exit_code)

    def question(self, message: str, level: str = "QUESTION") -> str:
        prompt = f"{self.prefix}{Fore.LIGHTCYAN_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}"
        print(prompt, end="")
        answer = input()
        self._write_to_log(prompt)
        self._write_to_log(f"User Answer: {answer}")
        return answer

    def message(self, message: str, start=None, end=None, level: str = "MESSAGE") -> None:
        msg = f"{self.prefix}{Fore.LIGHTMAGENTA_EX}{level:<8}{Fore.BLACK}➔{Fore.RESET} {message}{self._timer_str(start, end)}"
        self._print_and_log(msg)



from setuptools import setup, find_packages

setup(
    name="logkaren",
    version="1.0.0",
    description="A beautiful, colorful Python logging library by karenhoyoshi",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="karenhoyoshi",
    url="https://github.com/karenhoyoshi/logkaren",
    packages=find_packages(),
    python_requires=">=3.10",
    install_requires=[
        "colorama>=0.4.6",
        "pyfiglet>=0.8.post1",
    ],
    extras_require={
        "dev": ["build", "twine", "pytest"],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Topic :: System :: Logging",
        "Environment :: Console",
    ],
    keywords="logging logger colorful terminal cli console",
    license="MIT",
)
